// +build ignore

package main

import (
	"io/ioutil"
	"os"
	"text/template"

	"github.com/elliotchance/pie/functions"
)

var packageTemplate = template.Must(template.New("").
	Parse("// Code generated by go generate; DO NOT EDIT.\n" +
		"package main\n" +
		"\n" +
		"var pieTemplates = map[string]string{\n" +
		"{{ range $fn, $file := . }}" +
		"\t\"{{ $fn }}\": `{{ $file }}`,\n" +
		"{{ end }}" +
		"}\n"))

func main() {
	data := map[string]string{}

	for _, function := range functions.Functions {
		tmpl, err := ioutil.ReadFile("functions/" + function.File)
		if err != nil {
			panic(err)
		}

		data[function.Name] = string(tmpl)
	}

	f, err := os.Create("template.go")
	if err != nil {
		panic(err)
	}
	defer f.Close()

	packageTemplate.Execute(f, data)
}
